@@ -1,6 +1,7 @@
 /* actions.c
  *
- * Copyright  2002 Lutz Mller <lutz@users.sourceforge.net>
+ * Copyright 2002 Lutz Mueller <lutz@users.sourceforge.net>
+ * Copyright 2016 Marcus Meissner <marcus@jet.franken.de>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -23,7 +24,11 @@
 #define __EXTENSIONS__	/* for solaris to get back strdup and strcasecmp */
 
 #include "config.h"
+#if defined (HAVE_SIGNAL_H)
+#include <signal.h>
+#endif
 
+#include <libgen.h>
 #include <string.h>
 #include <stdio.h>
 #ifdef HAVE_FCNTL_H
@@ -705,6 +710,10 @@ action_camera_show_abilities (GPParams *p)
 		printf (_("                                 : Preview\n"));
 		has_capture = 1;
 	}
+	if (a.operations & GP_OPERATION_TRIGGER_CAPTURE) {
+		printf (_("                                 : Trigger Capture\n"));
+		has_capture = 1;
+	}
 	if (has_capture == 0) {
 		printf (_("                                 : Capture not supported by the driver\n"));
 	}
@@ -902,7 +911,7 @@ print_version_action (GPParams __unused__ *p)
 #endif
 	printf (_("gphoto2 %s\n"
 		  "\n"
-		  "Copyright (c) 2000-%d Lutz Mueller and others\n"
+		  "Copyright (c) 2000-%d Marcus Meissner and others\n"
 		  "%s"
 		  "\n"
 		  "gphoto2 comes with NO WARRANTY, to the extent permitted by law. You may\n"
@@ -911,7 +920,7 @@ print_version_action (GPParams __unused__ *p)
 		  "\n"
 		  "This version of gphoto2 is using the following software versions and options:\n"),
 		VERSION,
-		2014, /* year of release! */
+		2020, /* year of release! */
 		port_message
 		);
 
@@ -938,46 +947,67 @@ print_version_action (GPParams __unused__ *p)
 	return (GP_OK);
 }
 
-int
-action_camera_capture_preview (GPParams *p)
+static int
+_action_camera_capture_preview (GPParams *p, int viewasciiart)
 {
 	CameraFile *file;
 	int	r, fd;
-        char    tmpname[20], *tmpfilename;
-
-	strcpy (tmpname, "tmpfileXXXXXX");
-	fd = mkstemp(tmpname);
-	if (fd == -1) {
-		CR (gp_file_new (&file));
-		tmpfilename = NULL;
+	char tmpname[20], *tmpfilename = NULL;
+	
+	if (p->flags & FLAGS_STDOUT) {
+		fd = dup(fileno(stdout));
+		CR (gp_file_new_from_fd (&file, fd));
 	} else {
-		r = gp_file_new_from_fd (&file, fd);
-		if (r < GP_OK) {
-			close (fd);
-			unlink (tmpname);
-			return r;
+		strcpy (tmpname, "tmpfileXXXXXX");
+		fd = mkstemp(tmpname);
+		if (fd == -1) {
+			CR (gp_file_new (&file));
+			tmpfilename = NULL;
+		} else {
+			r = gp_file_new_from_fd (&file, fd);
+			if (r < GP_OK) {
+				close (fd);
+				unlink (tmpname);
+				return r;
+			}
+			tmpfilename = tmpname;
 		}
-		tmpfilename = tmpname;
 	}
 
 #ifdef HAVE_AA
-	r = gp_cmd_capture_preview (p->camera, file, p->context);
-#else
-	r = gp_camera_capture_preview (p->camera, file, p->context);
+	if (viewasciiart)
+		r = gp_cmd_capture_preview (p->camera, file, p->context);
+	else
 #endif
+		r = gp_camera_capture_preview (p->camera, file, p->context);
+	fflush(stdout);
 	if (r < 0) {
-		gp_file_unref (file);
+		if(!(p->flags & FLAGS_STDOUT))
+			gp_file_unref (file);
 		unlink (tmpname);
-		return (r);
+		return r;
 	}
 
-	r = save_camera_file_to_file (NULL, "capture_preview", GP_FILE_TYPE_NORMAL, file, tmpfilename);
-	gp_file_unref (file);
-	if (r < 0) {
-		unlink (tmpname);
-		return (r);
+	/* name it file_%filename if --filename is set, otherwise capture_preview */
+	if(!(p->flags & FLAGS_STDOUT)) {
+		r = save_camera_file_to_file (NULL, "capture_preview", p->filename?GP_FILE_TYPE_PREVIEW:GP_FILE_TYPE_NORMAL, file, tmpfilename);
+		gp_file_unref (file);
+		if (r < 0) {
+			unlink (tmpname);
+			return (r);
+		}
 	}
-	return (GP_OK);
+	return GP_OK;
+}
+
+int
+action_camera_capture_preview (GPParams *p) {
+	return _action_camera_capture_preview (p, 0);
+}
+
+int
+action_camera_show_preview (GPParams *p) {
+	return _action_camera_capture_preview (p, 1);
 }
 
 enum moviemode { MOVIE_ENDLESS, MOVIE_FRAMES, MOVIE_SECONDS };
@@ -986,14 +1016,14 @@ int
 action_camera_capture_movie (GPParams *p, const char *arg)
 {
 	CameraFile	*file;
-	int		r;
+	int		r, tries;
 	int		fd;
 	time_t		st;
 	enum moviemode	mm;
 	int		frames,captured_frames=0;
 	char		*xname;
 	struct timeval	starttime;
-
+	
 	if (p->flags & FLAGS_STDOUT) {
 		fd = dup(fileno(stdout));
 		xname = "stdout";
@@ -1022,13 +1052,20 @@ action_camera_capture_movie (GPParams *p, const char *arg)
 	}
 	CR (gp_file_new_from_fd (&file, fd));
 	gettimeofday (&starttime, NULL);
+	tries = 0;
 	while (1) {
 		const char *mime;
 		r = gp_camera_capture_preview (p->camera, file, p->context);
 		if (r < 0) {
+			if (r == GP_ERROR_CAMERA_BUSY) {
+				/* allow 20 busy tries */
+				if (tries++ < 20)
+					continue; /* just continue */
+			}
 			cli_error_print(_("Movie capture error... Exiting."));
 			break;
 		}
+		tries = 0;
 		gp_file_get_mime_type (file, &mime);
                 if (strcmp (mime, GP_MIME_JPEG)) {
 			cli_error_print(_("Movie capture error... Unhandled MIME type '%s'."), mime);
@@ -1076,12 +1113,14 @@ action_camera_wait_event (GPParams *p, enum download_type downloadtype, const ch
 	struct timeval	xtime;
 	int events, frames;
 
+        end_next = 0;
+
 	gettimeofday (&xtime, NULL);
 	memset(&last,0,sizeof(last));
 
 	wp.type = WAIT_EVENTS;
 	wp.u.events = 1000000;
-	if (!arg) {
+	if (!arg || !strlen(arg)) {
 		printf ( _("Waiting for events from camera. Press Ctrl-C to abort.\n"));
 	} else {
 		int x;
@@ -1105,7 +1144,7 @@ action_camera_wait_event (GPParams *p, enum download_type downloadtype, const ch
 		} else {
 			wp.type = WAIT_STRING;
 			wp.u.str = arg;
-			printf ( _("Waiting for %s event from camera. Press Ctrl-C to abort.\n"), wp.u.str);
+			printf ( _("Waiting for '%s' event from camera. Press Ctrl-C to abort.\n"), wp.u.str);
 		}
 	}
 
@@ -1117,6 +1156,12 @@ action_camera_wait_event (GPParams *p, enum download_type downloadtype, const ch
 
 		if (glob_cancel) break;
 
+		if (end_next) {
+			printf(_("SIGUSR2 signal received, stopping wait!\n"));
+			end_next = 0;
+			break;
+		}
+
 		exitloop = 0;
 		switch (wp.type) {
 		case WAIT_STRING:
@@ -1159,6 +1204,10 @@ action_camera_wait_event (GPParams *p, enum download_type downloadtype, const ch
 			break;
 		case GP_EVENT_TIMEOUT:
 			/*printf("TIMEOUT\n");*/
+			if ((wp.type == WAIT_STRING) && strstr("TIMEOUT",wp.u.str)) {
+				printf(_("event found, stopping wait!\n"));
+				return GP_OK;
+			}
 			break;
 		case GP_EVENT_CAPTURE_COMPLETE:
 			printf("CAPTURECOMPLETE\n");
@@ -1172,7 +1221,11 @@ action_camera_wait_event (GPParams *p, enum download_type downloadtype, const ch
 
 			fn = (CameraFilePath*)data;
 
-			if (downloadtype == DT_NO_DOWNLOAD) {
+			if (	(downloadtype == DT_NO_DOWNLOAD)	||
+				(	(p->flags & FLAGS_KEEP_RAW) &&
+				 	( !strstr(fn->name,".jpg") && !strstr(fn->name,".JPG")) 
+				)
+			) {
 				printf("FILEADDED %s %s\n",fn->name, fn->folder);
 				if ((wp.type == WAIT_STRING) && strstr("FILEADDED",wp.u.str)) {
 					printf(_("event found, stopping wait!\n"));
@@ -1225,6 +1278,14 @@ action_camera_wait_event (GPParams *p, enum download_type downloadtype, const ch
 				return GP_OK;
 			}
 			break;
+		case GP_EVENT_FILE_CHANGED:
+			fn = (CameraFilePath*)data;
+			printf("FILECHANGED %s %s\n",fn->name, fn->folder);
+			if ((wp.type == WAIT_STRING) && strstr("FILECHANGED",wp.u.str)) {
+				printf(_("event found, stopping wait!\n"));
+				return GP_OK;
+			}
+			break;
 		}
 		free (data);
 	}
@@ -1317,6 +1378,9 @@ print_storage_info (GPParams *p)
 		if (sinfos[i].fields & GP_STORAGEINFO_FREESPACEIMAGES)
 			printf ("freeimages=%lu\n", (unsigned long)sinfos[i].freeimages);
 	}
+	if (sinfos)
+		free(sinfos);
+
 	return GP_OK;
 }
 
@@ -1522,6 +1586,13 @@ static int
 _find_widget_by_name (GPParams *p, const char *name, CameraWidget **child, CameraWidget **rootconfig) {
 	int	ret;
 
+	*rootconfig = NULL;
+	ret = gp_camera_get_single_config (p->camera, name, child, p->context);
+	if (ret == GP_OK) {
+		*rootconfig = *child;
+		return GP_OK;
+	}
+
 	ret = gp_camera_get_config (p->camera, rootconfig, p->context);
 	if (ret != GP_OK) return ret;
 	ret = gp_widget_get_child_by_name (*rootconfig, name, child);
@@ -1550,19 +1621,19 @@ _find_widget_by_name (GPParams *p, const char *name, CameraWidget **child, Camer
 			if (ret != GP_OK)
 				break;
 			*child = tmp;
-			if (!s) /* end of path */
-				break;
+			if (!s) {
+				/* end of path */
+				free (newname);
+				return GP_OK;
+			}
 			part = s+1;
 			while (part[0] == '/')
 				part++;
 		}
-		if (s) { /* if we have stuff left over, we failed */
-			gp_context_error (p->context, _("%s not found in configuration tree."), newname);
-			free (newname);
-			gp_widget_free (*rootconfig);
-			return GP_ERROR;
-		}
+		gp_context_error (p->context, _("%s not found in configuration tree."), newname);
 		free (newname);
+		gp_widget_free (*rootconfig);
+		return GP_ERROR;
 	}
 	return GP_OK;
 }
@@ -1587,7 +1658,7 @@ static int
 print_widget (GPParams *p, const char *name, CameraWidget *widget) {
 	const char *label;
 	CameraWidgetType	type;
-	int ret;
+	int ret, readonly;
 
 	ret = gp_widget_get_type (widget, &type);
 	if (ret != GP_OK)
@@ -1595,8 +1666,13 @@ print_widget (GPParams *p, const char *name, CameraWidget *widget) {
 	ret = gp_widget_get_label (widget, &label);
 	if (ret != GP_OK)
 		return ret;
+		
+	ret = gp_widget_get_readonly (widget, &readonly);
+	if (ret != GP_OK)
+		return ret;
 
 	printf ("Label: %s\n", label); /* "Label:" is not i18ned, the "label" variable is */
+	printf ("Readonly: %d\n", readonly);
 	switch (type) {
 	case GP_WIDGET_TEXT: {		/* char *		*/
 		char *txt;
@@ -1689,6 +1765,8 @@ print_widget (GPParams *p, const char *name, CameraWidget *widget) {
 	case GP_WIDGET_BUTTON:
 		break;
 	}
+	
+	printf ("END\n");
 	return GP_OK;
 }
 
@@ -1876,7 +1954,10 @@ set_config_action (GPParams *p, const char *name, const char *value) {
 		break;
 	}
 	if (ret == GP_OK) {
-		ret = gp_camera_set_config (p->camera, rootconfig, p->context);
+		if (child == rootconfig)
+			ret = gp_camera_set_single_config (p->camera, name, child, p->context);
+		else
+			ret = gp_camera_set_config (p->camera, rootconfig, p->context);
 		if (ret != GP_OK)
 			gp_context_error (p->context, _("Failed to set new configuration value %s for configuration entry %s."), value, name);
 	}
@@ -1944,7 +2025,10 @@ set_config_index_action (GPParams *p, const char *name, const char *value) {
 		break;
 	}
 	if (ret == GP_OK) {
-		ret = gp_camera_set_config (p->camera, rootconfig, p->context);
+		if (child == rootconfig)
+			ret = gp_camera_set_single_config (p->camera, name, child, p->context);
+		else
+			ret = gp_camera_set_config (p->camera, rootconfig, p->context);
 		if (ret != GP_OK)
 			gp_context_error (p->context, _("Failed to set new configuration value %s for configuration entry %s."), value, name);
 	}
@@ -2028,8 +2112,10 @@ set_config_value_action (GPParams *p, const char *name, const char *value) {
 		int	t = -1;
 		struct tm xtm;
 
+		if (!strcasecmp (value, "now")  || !strcasecmp (value, _("now")))
+			t = time(NULL);
 #ifdef HAVE_STRPTIME
-		if (strptime (value, "%c", &xtm) || strptime (value, "%Ec", &xtm))
+		else if (strptime (value, "%c", &xtm) || strptime (value, "%Ec", &xtm))
 			t = mktime (&xtm);
 #endif
 		if (t == -1) {
@@ -2084,7 +2170,10 @@ set_config_value_action (GPParams *p, const char *name, const char *value) {
 		break;
 	}
 	if (ret == GP_OK) {
-		ret = gp_camera_set_config (p->camera, rootconfig, p->context);
+		if (child == rootconfig)
+			ret = gp_camera_set_single_config (p->camera, name, child, p->context);
+		else
+			ret = gp_camera_set_config (p->camera, rootconfig, p->context);
 		if (ret != GP_OK)
 			gp_context_error (p->context, _("Failed to set new configuration value %s for configuration entry %s."), value, name);
 	}
