@@ -96,10 +96,10 @@ static int glob_bulblength = 0;
 GPParams gp_params;
 
 /* flag for SIGUSR1 handler */
-static volatile int capture_now = 0;
+volatile int capture_now = 0;
 
 /* flag for SIGUSR2 handler */
-static volatile int end_next = 0;
+volatile int end_next = 0;
 
 /*! \brief Copy string almost like strncpy, converting to lower case.
  *
@@ -910,6 +910,11 @@ wait_and_handle_event (long waittime, CameraEventType *type, int download) {
 			printf (_("Event FOLDER_ADDED %s/%s during wait, ignoring.\n"), path->folder, path->name);
 		free (data);
 		break;
+	case GP_EVENT_FILE_CHANGED:
+		if (!(gp_params.flags & FLAGS_QUIET))
+			printf (_("Event FILE_CHANGED %s/%s during wait, ignoring.\n"), path->folder, path->name);
+		free (data);
+		break;
 	case GP_EVENT_FILE_ADDED:
 		result = save_captured_file (path, download);
 		free (data);
@@ -963,11 +968,6 @@ capture_generic (CameraCaptureType type, const char __unused__ *name, int downlo
 		}
 	}
 
-	capture_now = 0;
-	signal(SIGUSR1, sig_handler_capture_now);
-	end_next = 0;
-	signal(SIGUSR2, sig_handler_end_next);
-
 	while(++frames) {
 		if (!(gp_params.flags & FLAGS_QUIET) && glob_interval) {
 			if(!glob_frames)
@@ -1067,6 +1067,7 @@ capture_generic (CameraCaptureType type, const char __unused__ *name, int downlo
 		 */
 		if (glob_interval != -1) {
 			waittime = timediff_now (&next_pic_time);
+			result = GP_OK;
 			if (waittime > 0) {
 				if (!(gp_params.flags & FLAGS_QUIET) && glob_interval)
 					printf (_("Waiting for next capture slot %ld seconds...\n"), waittime/1000);
@@ -1093,6 +1094,7 @@ capture_generic (CameraCaptureType type, const char __unused__ *name, int downlo
 				if (!(gp_params.flags & FLAGS_QUIET) && glob_interval)
 					printf (_("not sleeping (%ld seconds behind schedule)\n"), -waittime/1000);
 			}
+			if (result != GP_OK) break;
 			if (capture_now && (gp_params.flags & FLAGS_RESET_CAPTURE_INTERVAL)) {
 				gettimeofday (&next_pic_time, NULL);
 				next_pic_time.tv_sec += glob_interval;
@@ -1147,9 +1149,7 @@ capture_generic (CameraCaptureType type, const char __unused__ *name, int downlo
 			gettimeofday (&expose_end_time, NULL);
 		}
 	}
-
-	signal(SIGUSR1, SIG_DFL);
-	return (GP_OK);
+	return GP_OK;
 }
 
 
@@ -1546,7 +1546,6 @@ cb_arg_init (poptContext __unused__ ctx,
 	case ARG_VERSION:
 		params->p.r = print_version_action (&gp_params);
 		break;
-
 	default:
 		break;
 	}
@@ -2187,6 +2186,12 @@ main (int argc, char **argv, char **envp)
         bindtextdomain (PACKAGE, LOCALEDIR);
         textdomain (PACKAGE);
 
+	/* These are for people signaling us */
+	capture_now = 0;
+	signal(SIGUSR1, sig_handler_capture_now);
+	end_next = 0;
+	signal(SIGUSR2, sig_handler_end_next);
+
 	/* Create/Initialize the global variables before we first use
 	 * them. And the signal handlers and popt callback functions
 	 * do use them. */
